@@ -1,7 +1,6 @@
 /*
  * invoking periodical RTC interrupts and report the stack pointers
  * 
- * for i386 only!
  */
 
 #include <linux/config.h>
@@ -89,10 +88,10 @@ static inline void copy_stack(int pcnt)
 	}
 	memcpy(&irq_info.comm[pcnt][0], current->comm, 16);
 }
-#elif defined(__ppc__)
+#elif defined(__powerpc__)
 static inline void copy_stack(int pcnt)
 {
-	unsigned long sp, stack_top, prev_sp, ret;
+	unsigned long sp, stack_top, prev_sp;
 	int i;
 	unsigned long *dump = &irq_info.stacks[pcnt][0];
 	unsigned short idx;
@@ -130,8 +129,10 @@ static void my_interrupt(void *private_data)
 
 	spin_lock(&my_lock);
 	count++;
-	if (count < irq_count)
+	if (count < irq_count) {
+		spin_unlock(&my_lock);
 		return;
+	}
 	count = 0;
 	pcnt = irq_info.processed;
 	irq_info.processed++;
@@ -144,9 +145,13 @@ static void my_interrupt(void *private_data)
 
 static int my_start(void)
 {
+	int	rc;
+
 	if (irq_count) {
-		rtc_control(&rtc_task, RTC_IRQP_SET, rtc_freq);
-		rtc_control(&rtc_task, RTC_PIE_ON, 0);
+		rc = rtc_control(&rtc_task, RTC_IRQP_SET, rtc_freq);
+		if (rc != 0) return rc;
+		rc = rtc_control(&rtc_task, RTC_PIE_ON, 0);
+		if (rc != 0) return rc;
 		rtc_running = 1;
 		return 0;
 	}
