@@ -37,6 +37,25 @@ static FILE *profile_fd;
 
 static inline unsigned long long int rdtsc(void)
 {
+#ifdef USE_GENERIC_TIMER
+	static struct timeval mytv;
+	static struct timezone tz;
+	gettimeofday(&mytv,&tz);
+	return(mytv.tv_sec*1000000+mytv.tv_usec);
+#elif USE_PPC_VEA_TIMEBASE
+	unsigned long x, y, z;
+
+        __asm __volatile (      "1:\n"
+				" mftbu %0\n"
+                                " mftb  %1\n"
+                                " mftbu %2\n"
+                                " cmpw  %0,%2\n"
+                                " bne   1b\n"
+                                " mftb  %1\n"
+                                : "=r" (x), "=r" (y), "=r" (z)
+                                : );
+        return ((((unsigned long long int) z)<<32) | y) ;
+#else
 	unsigned long long int x, y;
 	for (;;) {
 		__asm__ volatile ("rdtsc" : "=A" (x));
@@ -44,8 +63,28 @@ static inline unsigned long long int rdtsc(void)
 		if (y - x < 1000)
 			return y;
 	}
+#endif
 }
 
+#if USE_PPC_VEA_TIMEBASE
+static double tb_ticks_per_sec = 0.0;
+
+static inline void calibrate_tb(void)
+{
+	struct timeval tv1, tv2 ;
+	unsigned long long int tb1, tb2 ;
+	double t1, t2 ;
+	gettimeofday(&tv1, NULL) ;
+	tb1 = rdtsc() ;
+	sleep(5) ;
+	gettimeofday(&tv2, NULL) ;
+	tb2 = rdtsc() ;
+	t1 = tv1.tv_sec + tv1.tv_usec/1000000.0 ;
+	t2 = tv2.tv_sec + tv2.tv_usec/1000000.0 ;	
+	tb_ticks_per_sec = (tb2-tb1)/(t2-t1) ;
+}
+#endif
+
 static unsigned long long time_offset;
 
 #define mygettime() (rdtsc() - time_offset)
@@ -56,7 +95,13 @@ static double cpu_hz;
 
 inline static double wallclock(unsigned long long sc)
 {
+#ifdef USE_GENERIC_TIMER
+	return (double)sc / 1000000.0;
+#elif USE_PPC_VEA_TIMEBASE
+	return (double)sc / tb_ticks_per_sec;
+#else
 	return (double)sc / cpu_hz;
+#endif
 }
 
 static int init_test(const char *device_file, int freq, int count, int use_rtc);
@@ -118,6 +163,9 @@ int main(int argc,char **argv)
 
 
 	time_offset = rdtsc(); /* initialization */
+#if USE_PPC_VEA_TIMEBASE
+	calibrate_tb();
+#endif
 
 	use_rtc = 0;
 	frequency = 1024;
@@ -130,6 +178,7 @@ int main(int argc,char **argv)
 		switch (c) {
 		case 'I':
 			use_rtc = 1;
+			wakeup_count = 1;
 			break;
 		case 'c':
 			cpu_load = atof(optarg);
@@ -223,7 +272,11 @@ int main(int argc,char **argv)
 	stinfo.start_time = wallclock(start_sc);
 	fprintf(stderr, "start time = %g\n", stinfo.start_time);
 
-	start_trigger(use_rtc);
+	err = start_trigger(use_rtc);
+	if (err < 0) {
+		fprintf(stderr, "fatal ioctl error %d\n", -err);
+		exit(1);
+	}
 	for (;;) {
 		int nrun = stinfo.num_runs;
 		unsigned long long time_head, time_cpu, time_sync;
@@ -347,7 +400,8 @@ static int init_test(const char *device_file, int frequency, int wakeup_count, i
 	} else {
 		devfd = open(device_file, O_RDWR);
 		if (devfd < 0) {
-			fprintf(stderr, "error opening device\n");
+			fprintf(stderr, "error opening test device\n");
+			perror("open");
 			return -errno;
 		}
 
@@ -367,7 +421,7 @@ static int init_test(const char *device_file, int frequency, int wakeup_count, i
 inline static int start_trigger(int use_rtc)
 {
 	if (use_rtc)
-		return ioctl(devfd, RTC_PIE_ON);
+		return ioctl(devfd, RTC_PIE_ON, 0);
 	else
 		return ioctl(devfd, LAT_TEST_START, 0);
 }
@@ -416,11 +470,15 @@ inline static int get_sync(struct latency_test_info *info, int use_rtc)
 		struct pollfd pfd;
 		unsigned long t;
 		pfd.fd = devfd;
-		pfd.fd = POLLIN;
-		if (poll(&pfd, 1, -1) < 0)
+		pfd.events = POLLIN;
+		if (poll(&pfd, 1, -1) < 0) {
+			perror("poll");
 			return -errno;
-		if (read(devfd, &t, sizeof(t)) < 0)
+		}
+		if (read(devfd, &t, sizeof(t)) < 0) {
+			perror("read");
 			return -errno;
+		}
 	} else {
 		if (ioctl(devfd, LAT_TEST_READ, info) < 0)
 			return -errno;
@@ -492,6 +550,14 @@ static int calibrate_loop(void)
 			cpu_hz=atof(&s1[10])*1000000.0;
 			break;
 		}
+		if(!memcmp(s1,"clock",5))
+		{
+			char *colon;
+			colon = index(s1,':');
+			if( colon == NULL ) break;
+			sscanf(colon+1,"%lf",&cpu_hz);
+			cpu_hz *= 1000000.0;
+		}
 	}
 	fclose(f);
 	if(cpu_hz < 1.0) {
