@@ -165,7 +165,7 @@ int main(int argc, char **argv)
 	float threshold = 0;
 	static char *ksymfile = "/proc/kallsyms";
 
-	while ((c = getopt(argc, argv, "k:hpr:)) != -1) {
+	while ((c = getopt(argc, argv, "k:hpr:")) != -1) {
 		switch (c) {
 		case 'k':
 			ksymfile = optarg;
